@@ -1,22 +1,21 @@
 ###__PERLBIN__###
-#  Copyright (C) 2002-2004 Adrian Ulrich <pab at blinkenlights.ch>
+#  Copyright (C) 2002-2007 Adrian Ulrich <pab at blinkenlights.ch>
 #  Part of the gnupod-tools collection
 #
 #  URL: http://www.gnu.org/software/gnupod/
 #
-#    This program is free software; you can redistribute it and/or modify
+#    GNUpod is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation; either version 2 of the License, or
+#    the Free Software Foundation; either version 3 of the License, or
 #    (at your option) any later version.
 #
-#    This program is distributed in the hope that it will be useful,
+#    GNUpod is distributed in the hope that it will be useful,
 #    but WITHOUT ANY WARRANTY; without even the implied warranty of
 #    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #    GNU General Public License for more details.
 #
 #    You should have received a copy of the GNU General Public License
-#    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.#
 #
 # iTunes and iPod are trademarks of Apple
 #
@@ -38,24 +37,25 @@ print "gnupod_check.pl Version ###__VERSION__### (C) Adrian Ulrich\n";
 $opts{mount} = $ENV{IPOD_MOUNTPOINT};
 #Don't add xml and itunes opts.. we *NEED* the mount opt to be set..
 GetOptions(\%opts, "version", "help|h", "mount|m=s", "fixit");
-GNUpod::FooBar::GetConfig(\%opts, {mount=>'s'}, "gnupod_check");
+GNUpod::FooBar::GetConfig(\%opts, {mount=>'s', 'automktunes'=>'b', model=>'s'}, "gnupod_check");
 
 usage() if $opts{help};
 version() if $opts{version};
 
+# Need this global :-(
+my $con = GNUpod::FooBar::connect(\%opts);
 go();
 
 ####################################################
 # Worker
 sub go {
-	my $con = GNUpod::FooBar::connect(\%opts);
 	usage($con->{status}."\n") if $con->{status};
 	
 	print "Pass 1: Checking Files in the GNUtunesDB.xml...\n";
 	GNUpod::XMLhelper::doxml($con->{xml}) or usage("Failed to parse $con->{xml}, did you run gnupod_INIT.pl?\n");
 
 	print "Pass 2: Checking Files on the iPod...\n";
-	checkGNUtunes();
+	checkGNUtunes($con);
 
 	print   "..finished\n\n";
 	print   "  Total Playtime : ".int($TRACKER{TIME}/1000/60/60)." h\n";
@@ -67,7 +67,7 @@ sub go {
 		if($TRACKER{FIXED}) {
 			print " -> Writing new GNUtunesDB.xml, i fixed $TRACKER{FIXED} Errors.\n";
 			print "    You may have to re-run $0\n";
-			GNUpod::XMLhelper::writexml($con);
+			GNUpod::XMLhelper::writexml($con, {automktunes=>$opts{automktunes}});
 		}
 		else {
 			print " -> Nothing fixed, no need to rewrite the GNUtunesDB.xml\n";
@@ -79,6 +79,7 @@ sub go {
 		}
 		elsif($TRACKER{GLOBFILES} != $TRACKER{ITFILES}) {
 			print " -> The GNUtunesDB.xml is inconsistent. Please try to fix the errors or run $0 --fixit\n";
+			print "    (Note: --fixit removes 'stale' files on the iPod)\n";
 		}
 		
 		if($TRACKER{ERR} > 25) {
@@ -93,7 +94,8 @@ sub go {
 ############################################
 # Glob all files
 sub checkGNUtunes {
-	foreach my $file (bsd_glob($opts{mount}."/iPod_Control/Music/*/*", GLOB_NOSORT)) {
+	my($c) = @_;
+	foreach my $file (bsd_glob($c->{musicdir}."/*/*", GLOB_NOSORT)) {
 	next if -d $file;
 	$TRACKER{GLOBFILES}++;
 		unless($TRACKER{PATH}{lc($file)}) { #Hmpf.. file maybe not in the GNUtunesDB
@@ -152,7 +154,7 @@ sub newfile {
 		print "  ID $id has a long filename. Some iPods may refuse to play this file with recent firmware! Run $0 --fixit\n";
 		$TRACKER{ERR}++;
 		if($opts{fixit}) {
-			my($ipod_path, $real_path) = GNUpod::XMLhelper::getpath($opts{mount},$rp);
+			my($ipod_path, $real_path) = GNUpod::XMLhelper::getpath($con,$rp);
 			if($ipod_path) {
 				print " fixit: Renaming $rp into $real_path\n";
 				my $rename_result = rename($rp,$real_path);
@@ -226,7 +228,7 @@ gnupod_check.pl checks for 'lost' files
    -h, --help              display this help and exit
        --version           output version information and exit
    -m, --mount=directory   iPod mountpoint, default is \$IPOD_MOUNTPOINT
-       --fixit             Try to fixup some errors
+       --fixit             Try to fixup some errors (may delete 'lost' files)
 Report bugs to <bug-gnupod\@nongnu.org>
 EOF
 }
